==> pytest-bdd-0.2.tar.gz: pytest-bdd-0.2/pytest_bdd/feature.py <==
"""Feature.

The way of describing the behavior is based on Gherkin language, but a very
limited version. It doesn't support any parameter tables or any variables.
If the parametrization is needed to generate more test cases it can be done
on the fixture level of the pytest.

Syntax example:

    Scenario: Publishing the article
        Given I'm an author user
        And I have an article
        When I go to the article page
        And I press the publish button
        Then I should not see the error message
        And the article should be published  # Note: will query the database

:note: The "#" symbol is used for comments.
:note: There're no multiline steps, the description of the step must fit in
one line.
"""

from pytest_bdd.types import SCENARIO, GIVEN, WHEN, THEN


class FeatureError(Exception):
    """Feature parse error."""
    pass


# Global features dictionary
features = {}


STEP_PREFIXES = {
    'Scenario: ': SCENARIO,
    'Given ': GIVEN,
    'When ': WHEN,
    'Then ': THEN,
    'And ': None,  # Unknown step type
}

COMMENT_SYMBOLS = '#'


def get_step_type(line):
    """Detect step type by the beginning of the line.

    :param line: Line of the Feature file
    :return: SCENARIO, GIVEN, WHEN, THEN, or `None` if can't be detected.
    """
    for prefix in STEP_PREFIXES:
        if line.startswith(prefix):
            return STEP_PREFIXES[prefix]


def strip(line):
    """Remove leading and trailing whitespaces and comments.

    :param line: Line of the Feature file.
    :return: Stripped line.
    """
    try:
        line = line[:line.index(COMMENT_SYMBOLS)]
    except ValueError:
        pass
    return line.strip()


def remove_prefix(line):
    """Remove the step prefix (Scenario, Given, When, Then or And).

    :param line: Line of the Feature file.
    :return: Line without the prefix.
    """
    for prefix in STEP_PREFIXES:
        if line.startswith(prefix):
            return line[len(prefix):].strip()
    return line


class Feature(object):
    """Feature."""

    def __init__(self, filename):
        """Parse the feature file.

        :param filename: Relative path to the feature file.
        """
        self.scenarios = {}

        scenario = None
        mode = None
        prev_mode = None

        with open(filename, 'r') as f:
            content = f.read()
            for line in content.split('\n'):
                line = strip(line)
                if not line:
                    continue

                mode = get_step_type(line) or mode

                if mode == GIVEN and prev_mode not in (GIVEN, SCENARIO):
                    raise FeatureError('Given steps must be the first in withing the Scenario')

                if mode == WHEN and prev_mode not in (SCENARIO, GIVEN, WHEN):
                    raise FeatureError('When steps must be the first or follow Given steps')

                if mode == THEN and prev_mode not in (GIVEN, WHEN, THEN):
                    raise FeatureError('Then steps must follow Given or When steps')

                prev_mode = mode

                # Remove Given, When, Then, And
                line = remove_prefix(line)

                if mode == SCENARIO:
                    self.scenarios[line] = scenario = Scenario(line)
                else:
                    scenario.add_step(line)

    @classmethod
    def get_feature(cls, filename):
        feature = features.get(filename)
        if not feature:
            feature = Feature(filename)
            features[filename] = feature
        return feature


class Scenario(object):
    """Scenario."""

    def __init__(self, name):
        self.name = name
        self.steps = []

    def add_step(self, step):
        """Add step."""
        self.steps.append(step)

==> pytest-bdd-0.5.0.tar.gz: pytest-bdd-0.5.0/pytest_bdd/feature.py <==
"""Feature.

The way of describing the behavior is based on Gherkin language, but a very
limited version. It doesn't support any parameter tables.
If the parametrization is needed to generate more test cases it can be done
on the fixture level of the pytest.
The <variable> syntax can be used here to make a connection between steps and
it will also validate the parameters mentioned in the steps with ones
provided in the pytest parametrization table.

Syntax example:

    Scenario: Publishing the article
        Given I'm an author user
        And I have an article
        When I go to the article page
        And I press the publish button
        Then I should not see the error message
        And the article should be published  # Note: will query the database

:note: The "#" symbol is used for comments.
:note: There're no multiline steps, the description of the step must fit in
one line.

"""
import re  # pragma: no cover

from pytest_bdd.types import SCENARIO, GIVEN, WHEN, THEN  # pragma: no cover


class FeatureError(Exception):  # pragma: no cover
    """Feature parse error."""
    pass


# Global features dictionary
features = {}  # pragma: no cover


STEP_PREFIXES = {  # pragma: no cover
    'Scenario: ': SCENARIO,
    'Given ': GIVEN,
    'When ': WHEN,
    'Then ': THEN,
    'And ': None,  # Unknown step type
}

COMMENT_SYMBOLS = '#'  # pragma: no cover

STEP_PARAM_RE = re.compile('\<(.+?)\>')  # pragma: no cover


def get_step_type(line):
    """Detect step type by the beginning of the line.

    :param line: Line of the Feature file
    :return: SCENARIO, GIVEN, WHEN, THEN, or `None` if can't be detected.
    """
    for prefix in STEP_PREFIXES:
        if line.startswith(prefix):
            return STEP_PREFIXES[prefix]


def get_step_params(name):
    """Return step parameters."""
    params = STEP_PARAM_RE.search(name)
    if params:
        return params.groups()
    return ()


def strip(line):
    """Remove leading and trailing whitespaces and comments.

    :param line: Line of the Feature file.
    :return: Stripped line.
    """
    try:
        line = line[:line.index(COMMENT_SYMBOLS)]
    except ValueError:
        pass
    return line.strip()


def remove_prefix(line):
    """Remove the step prefix (Scenario, Given, When, Then or And).

    :param line: Line of the Feature file.

    :return: Line without the prefix.

    """
    for prefix in STEP_PREFIXES:
        if line.startswith(prefix):
            return line[len(prefix):].strip()
    return line


class Feature(object):
    """Feature."""

    def __init__(self, filename):
        """Parse the feature file.

        :param filename: Relative path to the feature file.

        """
        self.scenarios = {}

        scenario = None
        mode = None
        prev_mode = None

        with open(filename, 'r') as f:
            content = f.read()
            for line in content.split('\n'):
                line = strip(line)
                if not line:
                    continue

                mode = get_step_type(line) or mode

                if mode == GIVEN and prev_mode not in (GIVEN, SCENARIO):
                    raise FeatureError('Given steps must be the first in withing the Scenario')

                if mode == WHEN and prev_mode not in (SCENARIO, GIVEN, WHEN):
                    raise FeatureError('When steps must be the first or follow Given steps')

                if mode == THEN and prev_mode not in (GIVEN, WHEN, THEN):
                    raise FeatureError('Then steps must follow Given or When steps')

                prev_mode = mode

                # Remove Given, When, Then, And
                line = remove_prefix(line)

                if mode == SCENARIO:
                    self.scenarios[line] = scenario = Scenario(line)
                else:
                    scenario.add_step(line)

    @classmethod
    def get_feature(cls, filename):
        """Get a feature by the filename.

        :param filename: Filename of the feature file.

        :return: `Feature` instance from the parsed feature cache.

        :note: The features are parsed on the execution of the test and
            stored in the global variable cache to improve the performance
            when multiple scenarios are referencing the same file.

        """
        feature = features.get(filename)
        if not feature:
            feature = Feature(filename)
            features[filename] = feature
        return feature


class Scenario(object):
    """Scenario."""

    def __init__(self, name):
        self.name = name
        self.params = set()
        self.steps = []

    def add_step(self, step):
        """Add step."""
        self.params.update(get_step_params(step))
        self.steps.append(step)
